@@ -26,6 +26,7 @@
 #include <boost/archive/binary_oarchive.hpp>
 
 #include "auth-querycache.hh"
+#include "auth-zonecache.hh"
 #include "utility.hh"
 
 
@@ -66,7 +67,7 @@ bool UeberBackend::loadmodule(const string &name)
 
   void *dlib=dlopen(name.c_str(), RTLD_NOW);
 
-  if(dlib == NULL) {
+  if(dlib == nullptr) {
     g_log<<Logger::Error <<"Unable to load module '"<<name<<"': "<<dlerror() << endl;
     return false;
   }
@@ -78,7 +79,7 @@ bool UeberBackend::loadModules(const vector<string>& modules, const string& path
 {
   for (const auto& module: modules) {
     bool res;
-    if (module.find(".")==string::npos) {
+    if (module.find('.')==string::npos) {
       res = UeberBackend::loadmodule(path+"/lib"+module+"backend.so");
     } else if (module[0]=='/' || (module[0]=='.' && module[1]=='/') || (module[0]=='.' && module[1]=='.')) {
       // absolute or current path
@@ -94,7 +95,7 @@ bool UeberBackend::loadModules(const vector<string>& modules, const string& path
   return true;
 }
 
-void UeberBackend::go(void)
+void UeberBackend::go()
 {
   if (::arg().mustDo("consistent-backends")) {
     s_doANYLookupsOnly = true;
@@ -112,8 +113,8 @@ void UeberBackend::go(void)
 
 bool UeberBackend::getDomainInfo(const DNSName &domain, DomainInfo &di, bool getSerial)
 {
-  for(vector<DNSBackend *>::const_iterator i=backends.begin();i!=backends.end();++i)
-    if((*i)->getDomainInfo(domain, di, getSerial))
+  for(auto backend : backends)
+    if(backend->getDomainInfo(domain, di, getSerial))
       return true;
   return false;
 }
@@ -121,7 +122,7 @@ bool UeberBackend::getDomainInfo(const DNSName &domain, DomainInfo &di, bool get
 bool UeberBackend::createDomain(const DNSName &domain, const DomainInfo::DomainKind kind, const vector<ComboAddress> &masters, const string &account)
 {
   for(DNSBackend* mydb :  backends) {
-    if(mydb->createDomain(domain, kind, masters, account)) {
+    if (mydb->createDomain(domain, kind, masters, account)) {
       return true;
     }
   }
@@ -267,15 +268,33 @@ bool UeberBackend::getTSIGKeys(std::vector< struct TSIGKey > &keys)
 
 void UeberBackend::reload()
 {
-  for ( vector< DNSBackend * >::iterator i = backends.begin(); i != backends.end(); ++i )
+  for (auto & backend : backends)
   {
-    ( *i )->reload();
+    backend->reload();
   }
 }
 
+void UeberBackend::updateZoneCache() {
+  if (!g_zoneCache.isEnabled()) {
+    return;
+  }
+
+  vector<tuple<DNSName, int>> zone_indices;
+  g_zoneCache.setReplacePending();
+
+  for (vector<DNSBackend*>::iterator i = backends.begin(); i != backends.end(); ++i )
+  {
+    vector<DomainInfo> zones;
+    (*i)->getAllDomains(&zones, true);
+    for(auto& di: zones) {
+      zone_indices.push_back({std::move(di.zone), (int)di.id});  // this cast should not be necessary
+    }
+  }
+  g_zoneCache.replace(zone_indices);
+}
+
 void UeberBackend::rediscover(string *status)
 {
-  
   for ( vector< DNSBackend * >::iterator i = backends.begin(); i != backends.end(); ++i )
   {
     string tmpstr;
@@ -283,14 +302,16 @@ void UeberBackend::rediscover(string *status)
     if(status) 
       *status+=tmpstr + (i!=backends.begin() ? "\n" : "");
   }
+
+  updateZoneCache();
 }
 
 
 void UeberBackend::getUnfreshSlaveInfos(vector<DomainInfo>* domains)
 {
-  for ( vector< DNSBackend * >::iterator i = backends.begin(); i != backends.end(); ++i )
+  for (auto & backend : backends)
   {
-    ( *i )->getUnfreshSlaveInfos( domains );
+    backend->getUnfreshSlaveInfos( domains );
   }  
 }
 
@@ -298,12 +319,22 @@ void UeberBackend::getUnfreshSlaveInfos(vector<DomainInfo>* domains)
 
 void UeberBackend::getUpdatedMasters(vector<DomainInfo>* domains)
 {
-  for ( vector< DNSBackend * >::iterator i = backends.begin(); i != backends.end(); ++i )
+  for (auto & backend : backends)
   {
-    ( *i )->getUpdatedMasters( domains );
+    backend->getUpdatedMasters( domains );
   }
 }
 
+bool UeberBackend::inTransaction()
+{
+  for (auto* b : backends )
+  {
+    if(b->inTransaction())
+      return true;
+  }
+  return false;
+}
+
 bool UeberBackend::getAuth(const DNSName &target, const QType& qtype, SOAData* sd, bool cachedOk)
 {
   // A backend can respond to our authority request with the 'best' match it
@@ -312,20 +343,44 @@ bool UeberBackend::getAuth(const DNSName &target, const QType& qtype, SOAData* s
   // of them has a more specific zone but don't bother asking this specific
   // backend again for b.c.example.com., c.example.com. and example.com.
   // If a backend has no match it may respond with an empty qname.
-
   bool found = false;
   int cstat;
   DNSName shorter(target);
   vector<pair<size_t, SOAData> > bestmatch (backends.size(), make_pair(target.wirelength()+1, SOAData()));
   do {
+    int zoneId{-1};
+    if(cachedOk && g_zoneCache.isEnabled()) {
+      if (g_zoneCache.getEntry(shorter, zoneId)) {
+        // Zone exists in zone cache, directly look up SOA.
+        // XXX: this code path and the cache lookup below should be merged; but that needs the code path below to also use ANY.
+        // Or it should just also use lookup().
+        DNSZoneRecord zr;
+        lookup(QType(QType::SOA), shorter, zoneId, nullptr);
+        if (!get(zr)) {
+          // zone has somehow vanished
+          DLOG(g_log << Logger::Info << "Backend returned no SOA for zone '" << shorter.toLogString() << "', which it reported as existing " << endl);
+          continue;
+        }
+        if (zr.dr.d_name != shorter) {
+          throw PDNSException("getAuth() returned an SOA for the wrong zone. Zone '"+zr.dr.d_name.toLogString()+"' is not equal to looked up zone '"+shorter.toLogString()+"'");
+        }
+        sd->qname = zr.dr.d_name;
+        fillSOAData(zr, *sd);
+        // leave database handle in a consistent state
+        while (get(zr))
+          ;
+        goto found;
+      }
+      // zone does not exist, try again with shorter name
+      continue;
+    }
 
     d_question.qtype = QType::SOA;
     d_question.qname = shorter;
-    d_question.zoneId = -1;
+    d_question.zoneId = zoneId;
 
     // Check cache
     if(cachedOk && (d_cache_ttl || d_negcache_ttl)) {
-
       cstat = cacheHas(d_question,d_answers);
 
       if(cstat == 1 && !d_answers.empty() && d_cache_ttl) {
@@ -390,7 +445,7 @@ bool UeberBackend::getAuth(const DNSName &target, const QType& qtype, SOAData* s
         DLOG(g_log<<Logger::Error<<"add pos cache entry: "<<sd->qname<<endl);
         d_question.qtype = QType::SOA;
         d_question.qname = sd->qname;
-        d_question.zoneId = -1;
+        d_question.zoneId = zoneId;
 
         DNSZoneRecord rr;
         rr.dr.d_name = sd->qname;
@@ -422,8 +477,8 @@ bool UeberBackend::getSOAUncached(const DNSName &domain, SOAData &sd)
   d_question.qname=domain;
   d_question.zoneId=-1;
 
-  for(vector<DNSBackend *>::const_iterator i=backends.begin();i!=backends.end();++i)
-    if((*i)->getSOA(domain, sd)) {
+  for(auto backend : backends)
+    if(backend->getSOA(domain, sd)) {
       if(domain != sd.qname) {
         throw PDNSException("getSOA() returned an SOA for the wrong zone. Question: '"+domain.toLogString()+"', answer: '"+sd.qname.toLogString()+"'");
       }
@@ -448,16 +503,16 @@ bool UeberBackend::getSOAUncached(const DNSName &domain, SOAData &sd)
 
 bool UeberBackend::superMasterAdd(const string &ip, const string &nameserver, const string &account) 
 {
-  for(vector<DNSBackend *>::const_iterator i=backends.begin();i!=backends.end();++i)
-    if((*i)->superMasterAdd(ip, nameserver, account)) 
+  for(auto backend : backends)
+    if(backend->superMasterAdd(ip, nameserver, account)) 
       return true;
   return false;
 }
 
 bool UeberBackend::superMasterBackend(const string &ip, const DNSName &domain, const vector<DNSResourceRecord>&nsset, string *nameserver, string *account, DNSBackend **db)
 {
-  for(vector<DNSBackend *>::const_iterator i=backends.begin();i!=backends.end();++i)
-    if((*i)->superMasterBackend(ip, domain, nsset, nameserver, account, db))
+  for(auto backend : backends)
+    if(backend->superMasterBackend(ip, domain, nsset, nameserver, account, db))
       return true;
   return false;
 }
@@ -469,8 +524,8 @@ UeberBackend::UeberBackend(const string &pname)
     instances.push_back(this); // report to the static list of ourself
   }
 
-  d_negcached=0;
-  d_cached=0;
+  d_negcached=false;
+  d_cached=false;
   d_cache_ttl = ::arg().asNum("query-cache-ttl");
   d_negcache_ttl = ::arg().asNum("negquery-cache-ttl");
   d_qtype = 0;
@@ -533,21 +588,18 @@ void UeberBackend::addCache(const Question &q, vector<DNSZoneRecord> &&rrs)
   if(!d_cache_ttl)
     return;
 
-  unsigned int store_ttl = d_cache_ttl;
   for(const auto& rr : rrs) {
-   if (rr.dr.d_ttl < d_cache_ttl)
-     store_ttl = rr.dr.d_ttl;
    if (rr.scopeMask)
      return;
   }
 
-  QC.insert(q.qname, q.qtype, std::move(rrs), store_ttl, q.zoneId);
+  QC.insert(q.qname, q.qtype, std::move(rrs), d_cache_ttl, q.zoneId);
 }
 
 void UeberBackend::alsoNotifies(const DNSName &domain, set<string> *ips)
 {
-  for ( vector< DNSBackend * >::iterator i = backends.begin(); i != backends.end(); ++i )
-    (*i)->alsoNotifies(domain,ips);
+  for (auto & backend : backends)
+    backend->alsoNotifies(domain,ips);
 }
 
 UeberBackend::~UeberBackend()
@@ -564,7 +616,7 @@ void UeberBackend::lookup(const QType &qtype,const DNSName &qname, int zoneId, D
     throw PDNSException("We are stale, please recycle");
   }
 
-  DLOG(g_log<<"UeberBackend received question for "<<qtype.getName()<<" of "<<qname<<endl);
+  DLOG(g_log<<"UeberBackend received question for "<<qtype.toString()<<" of "<<qname<<endl);
   if (!d_go) {
     g_log<<Logger::Error<<"UeberBackend is blocked, waiting for 'go'"<<endl;
     std::unique_lock<std::mutex> l(d_mut);
@@ -616,9 +668,9 @@ void UeberBackend::lookup(const QType &qtype,const DNSName &qname, int zoneId, D
 }
 
 void UeberBackend::getAllDomains(vector<DomainInfo> *domains, bool include_disabled) {
-  for (vector<DNSBackend*>::iterator i = backends.begin(); i != backends.end(); ++i )
+  for (auto & backend : backends)
   {
-    (*i)->getAllDomains(domains, include_disabled);
+    backend->getAllDomains(domains, include_disabled);
   }
 }
 
@@ -682,9 +734,9 @@ UeberBackend::handle::handle()
 {
   //  g_log<<Logger::Warning<<"Handle instances: "<<instances<<endl;
   ++instances;
-  parent=NULL;
-  d_hinterBackend=NULL;
-  pkt_p=NULL;
+  parent=nullptr;
+  d_hinterBackend=nullptr;
+  pkt_p=nullptr;
   i=0;
   zoneId = -1;
 }
@@ -696,7 +748,7 @@ UeberBackend::handle::~handle()
 
 bool UeberBackend::handle::get(DNSZoneRecord &r)
 {
-  DLOG(g_log << "Ueber get() was called for a "<<qtype.getName()<<" record" << endl);
+  DLOG(g_log << "Ueber get() was called for a "<<qtype.toString()<<" record" << endl);
   bool isMore=false;
   while(d_hinterBackend && !(isMore=d_hinterBackend->get(r))) { // this backend out of answers
     if(i<parent->backends.size()) {
